"use strict";
// This file contains the bits and bobs of the internal API for loading and interacting with Nx plugins.
// For the public API, used by plugin authors, see `./public-api.ts`.
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultPlugins = exports.loadNxPlugin = exports.loadNxPlugins = exports.nxPluginCache = void 0;
const path_1 = require("path");
const workspace_root_1 = require("../../utils/workspace-root");
const angular_json_1 = require("../../adapter/angular-json");
const plugin_pool_1 = require("./plugin-pool");
// Short lived cache (cleared between cmd runs)
// holding resolved nx plugin objects.
// Allows loaded plugins to not be reloaded when
// referenced multiple times.
exports.nxPluginCache = new Map();
async function loadNxPlugins(plugins, root = workspace_root_1.workspaceRoot) {
    const result = [];
    plugins ??= [];
    plugins.unshift((0, path_1.join)(__dirname, '../../plugins/project-json/build-nodes/package-json-next-to-project-json'));
    // We push the nx core node plugins onto the end, s.t. it overwrites any other plugins
    plugins.push(...(await getDefaultPlugins(root)));
    for (const plugin of plugins) {
        result.push(loadNxPlugin(plugin, root));
    }
    return Promise.all(result);
}
exports.loadNxPlugins = loadNxPlugins;
async function loadNxPlugin(plugin, root = workspace_root_1.workspaceRoot) {
    const cacheKey = JSON.stringify(plugin);
    if (exports.nxPluginCache.has(cacheKey)) {
        return await exports.nxPluginCache.get(cacheKey);
    }
    const loadingPlugin = (0, plugin_pool_1.loadRemoteNxPlugin)(plugin, root);
    exports.nxPluginCache.set(cacheKey, loadingPlugin);
    return await loadingPlugin;
}
exports.loadNxPlugin = loadNxPlugin;
async function getDefaultPlugins(root) {
    return [
        (0, path_1.join)(__dirname, '../../plugins/js'),
        (0, path_1.join)(__dirname, '../../plugins/target-defaults/target-defaults-plugin'),
        ...((0, angular_json_1.shouldMergeAngularProjects)(root, false)
            ? [(0, path_1.join)(__dirname, '../../adapter/angular-json')]
            : []),
        (0, path_1.join)(__dirname, '../../plugins/package-json-workspaces'),
        (0, path_1.join)(__dirname, '../../plugins/project-json/build-nodes/project-json'),
    ];
}
exports.getDefaultPlugins = getDefaultPlugins;
